'''
/*****************************************************************************************************
*                                   NOTICE
*
* THIS COMPUTER SOFTWARE CONTAINS PROPRIETARY INFORMATION OF USDA-ARS/SPRU.
* NEITHER RECEIPT NOR POSSESSION THEREOF COVERS ANY RIGHT TO
* REPRODUCE OR USE, OR DISCLOSE IN WHOLE OR IN PART, ANY SUCH INFORMATION
* WITHOUT WRITTEN AUTHORIZATION FROM USDA-ARS/SPRU.  
* 
* ****************************************************************************************************
* Program: iStitch.py
* ****************************************************************************************************
* Purpose: GUI-based image stitching software
* Author: James Kim (james.y.kim@usda.gov)
* Version:  V1.0 2020-08-27  
* Revision: V1.1 2021-11-12
* Revision: V1.2 2024-04-06
* ****************************************************************************************************/
'''
path = '../Data/'

import os, time
import tkinter as tk #cf)Tkinter for Python 2.*
from tkinter import filedialog,simpledialog,ttk
import cv2
import numpy as np
from math import * 
from PIL import Image, ImageOps
from matplotlib import pyplot as plt
#Planet API: e14c0368bb8b48849078e6594159e880

#Local libraries
import Parse_Filenames as ParseFile
import LatLon2UTM as LL2UTM     # convert Lat\Lon to UTM
import Calculate_Heading as Heading
import RowDetection as RD

#stage = 2   # growth stage (1:Early,2:Mid,3:Late) for row detection algorithm
imW,imH = 1024,768
ppm = imW/4.064     # pixel per meter, 1024/(40'*4)=4.064

class Dialog_Input4Stitch(simpledialog.Dialog):
    def __init__(self,parent,defaultInputs, title=None, entry_to_clear=None):     
        tk.simpledialog.Dialog.__init__(self, parent, title) 
    def body(self, parent):
        stage,skew_GS,skew,Heading,Shifting,overlap,Trim,scale = defaultInputs

        #Add a checkbutton for open file
        self.OpenFile_var = tk.IntVar()
        tk.Checkbutton(parent,text='Open images',command=Open_Folder,variable=self.OpenFile_var,bd=4,indicatoron=False,onvalue=1,offvalue=0).grid(row=0,column=0,sticky=tk.W)

        #Add a checkbutton for Geo-calibration thru Row Detection
        self.GeoCal_var = tk.IntVar()
        tk.Checkbutton(parent,text='Geo-Calib,   Stage:',variable=self.GeoCal_var).grid(row=1,column=0,sticky=tk.W)
        self.GeoCal_var.set(0)
        # add a ComboBox for growth stage
        #tk.Label(parent, text="Stage: ").grid(row=1,sticky=tk.W)
        self.dic_Stage = {'Early':0, 'Mid':1, 'Late':2} # growth stage (1:Early,2:Mid,3:Late) for row detection algorithm
        self.comboboxStage = ttk.Combobox(parent, value = list(self.dic_Stage.keys()), width=4)
        self.comboboxStage.grid(row=1,column=1,sticky=tk.W)
        self.comboboxStage.current(0)        
        # add an Entry for skew angle in Geo-Calibration
        tk.Label(parent, text="Skew [deg]: ").grid(row=1,column=2,sticky=tk.W)
        self.e0 = tk.Entry(parent, width=4)
        self.e0.grid(row=1,column=3,sticky=tk.W)
        self.e0.insert(0,'%d'%skew_GC)  

        # add an Entry for skew angle
        tk.Label(parent, text="Skew angle [deg] : ").grid(row=2,sticky=tk.W)
        self.e1 = tk.Entry(parent, width=4)
        self.e1.grid(row=2,column=1,sticky=tk.W)
        self.e1.insert(0,'%d'%skew)  

        # add a ComboBox for heading direction
        tk.Label(parent, text="Heading, Keep : ").grid(row=3,sticky=tk.W)
        self.dic_Heading = {'N':0, 'E':1, 'S':2, 'W':3}
        self.comboboxHeading = ttk.Combobox(parent, value = list(self.dic_Heading.keys()), width=4)
        self.comboboxHeading.grid(row=3,column=1,sticky=tk.W)
        self.comboboxHeading.current(0)        
        # add a ComboBox for heading keep priority
        self.dic_HeadingKeep = {'Current':1, 'Next':2}
        self.comboboxHeadingKeep = ttk.Combobox(parent, value = list(self.dic_HeadingKeep.keys()), width=7)
        self.comboboxHeadingKeep.grid(row=3,column=2,sticky=tk.W)
        self.comboboxHeadingKeep.current(0)        

        #Add a checkbutton for Shifting option
        self.Shifting_var = tk.IntVar()
        tk.Checkbutton(parent,text='Shifting, Keep :',variable=self.Shifting_var).grid(row=4,column=0,sticky=tk.W)
        self.Shifting_var.set(1)
        # add a ComboBox for shifting direction
        self.comboboxShifting = ttk.Combobox(parent, value = list(self.dic_Heading.keys()), width=4)
        self.comboboxShifting.grid(row=4,column=1,sticky=tk.W)
        self.comboboxShifting.current(1)        
        # add a ComboBox for shifting keep priority
        self.dic_ShiftingKeep = {'Current':1, 'Next':2}
        self.comboboxShiftingKeep = ttk.Combobox(parent, value = list(self.dic_ShiftingKeep.keys()), width=7)
        self.comboboxShiftingKeep.grid(row=4,column=2,sticky=tk.W)
        self.comboboxShiftingKeep.current(0)        

        # add an Entry for trim % in top
        tk.Label(parent, text="Trim top [%] : ").grid(row=5,sticky=tk.W)
        self.e2 = tk.Entry(parent, width=4)
        self.e2.grid(row=5,column=1,sticky=tk.W)
        self.e2.insert(0,'%d'%overlap)  

        # add an Entry for trim % in left an right
        tk.Label(parent, text="Trim L&R [%]: ").grid(row=6,sticky=tk.W)
        self.e3 = tk.Entry(parent, width=8)
        self.e3.grid(row=6,column=1,sticky=tk.W)
        self.e3.insert(0,'%s, %s'%(Trim[0],Trim[1]))  

        # add an Entry for a resizing scale
        tk.Label(parent, text="Resize [x]: ").grid(row=7,sticky=tk.W)
        self.e4 = tk.Entry(parent, width=4)
        self.e4.grid(row=7,column=1,sticky=tk.W)
        self.e4.insert(0,'%.1f'%scale)  

        self.skew_GC = self.overlap = self.skew = self.scale = 0

    def apply(self):
        self.GeoCal_var = self.GeoCal_var.get()
        if self.comboboxStage.get()=="": self.stage = stage
        else: self.stage = self.comboboxStage.get() 
        if self.e0.get()=="": self.skew_GC = skew_GC
        else: self.skew_GC = float(self.e0.get())  
        if self.e1.get()=="": self.skew = skew
        else: self.skew = float(self.e1.get())  
        if self.e2.get()=="": self.overlap = overlap
        else: self.overlap = float(self.e2.get()) 
        if self.comboboxHeading.get()=="": self.heading = Heading[0]
        else: self.heading = self.comboboxHeading.get() 
        if self.comboboxHeadingKeep.get()=="": self.headingKeep = Heading[1]
        else: self.headingKeep = self.dic_HeadingKeep[self.comboboxHeadingKeep.get()] 
        self.Shift_var = self.Shifting_var.get()
        if self.comboboxShifting.get()=="": self.shifting = Shifting[0]
        else: self.shifting = self.comboboxShifting.get() 
        if self.comboboxShiftingKeep.get()=="": self.shiftingKeep = Shifting[1]
        else: self.shiftingKeep = self.dic_ShiftingKeep[self.comboboxShiftingKeep.get()] 
        #if self.e3.get()=="": self.scale = scale
        #else: self.scale = float(self.e3.get())  
        if self.e3.get()=="": self.trim = Trim
        else: self.trim = str(self.e3.get()).split(',')
        if self.e4.get()=="": self.scale = scale
        else: self.scale = float(self.e4.get())  
        # Convert the trim string to a list
        if len(self.trim)==1: self.trim = [float(self.trim[0]),0]
        else: self.trim = [float(self.trim[0]),float(self.trim[1])]
         
    #def cancel(self, event=None):  #NOTE: fix to skip cancel when clicking ok
        #self.destroy()
        #self.skew = None
         
#Open a folder in a directory
def Open_Folder():
    global src_dir
    src_dir = tk.filedialog.askdirectory() 
    return src_dir

#Load images in a directory
def load_images(src_dir):
    if not os.path.exists(src_dir):
        print('%s does not exist. Please enter a valid path.\n' % src_dir)

    Im = []
    print('Loading images...')
    for filename in os.listdir(src_dir):
        if filename.endswith('.tif') or filename.endswith('.jpg'):
            #print('filename=',filename)
            im = cv2.imread(os.path.join(src_dir,filename))
            Im.append(im)
            
    #Create image plot in calendar format
    print('total # of images =',len(Im))
    #im = np.array(Im[0])
    #cv2.imshow('im',im), cv2.waitKey(0)
    #im = np.array(Im[1])
    #cv2.imshow('im',im), cv2.waitKey(0)
    
    return Im

#Deskew images 
def deskew_images(Im,skew):
    print('Deskewing images...')
    imH, imW = Im[0].shape[:2]
    offsetX=int(tan(radians(skew))*imH)+260
    print('WxH=',imW,imH,'skew=',skew,'offsetX=',offsetX)

    Im_deskew = []
    for i,im in enumerate(Im):
        pts1 = np.float32(
            [[0, imH],     #LB
             [imW, imH],  #RB
             [imW*skew/90, 0], #LT
             [imW*(1-skew/90), 0]] #RT
        )
        pts2 = np.float32(
            [[0+offsetX, imH],
             [imW, imH],
             [0+offsetX,    0],
             [imW, 0]]
        )   

        # Get the Perspective Transform Matrix i.e. PT 
        PT = cv2.getPerspectiveTransform(pts1,pts2)
        
        # Apply the Perspective Transform to the src image
        transformed = np.zeros((int(imW+offsetX), imH), dtype=np.uint8);
        dst = cv2.warpPerspective(im, PT, transformed.shape)
        
        #plt.subplot(121),plt.imshow(im),plt.title('Input')
        #plt.subplot(122),plt.imshow(dst),plt.title('Output')
        #plt.show()
        Im_deskew.append(dst)
    
    return Im_deskew

def pil_grid(images, FD_W,FD_H, max_horiz=np.iinfo(int).max):
    n_images = len(images)
    n_horiz = min(n_images, max_horiz)
    h_sizes, v_sizes = [0] * n_horiz, [0] * int(ceil(n_images / float(n_horiz)))
    print('n_images={}, n_horiz={}, h_sizes={}, v_size={}'.format(n_images,n_horiz,h_sizes, v_sizes))
    for i, img in enumerate(images):
        h, v = i % n_horiz, i // n_horiz
        h_sizes[h] = max(h_sizes[h], img.size[0])
        v_sizes[v] = max(v_sizes[v], img.size[1])
        #print('h={},v={},h_sizes[h]={},v_sizes[v]={},img.size[0]={}'.format(h,v,h_sizes[h],v_sizes[v],img.size))
    h_sizes, v_sizes = np.cumsum([0] + h_sizes), np.cumsum([0] + v_sizes)
    print('h_sizes, v_sizes=',h_sizes, v_sizes)

    #img_grid = Image.new('RGB', (h_sizes[-1], v_sizes[-1]), color='white')
    img_grid = Image.new('RGB', (FD_W, FD_H), color='white')
    for i, img in enumerate(images):
        #print('h_sizes[{}]={}'.format(i%n_horiz,h_sizes[i%n_horiz]),'v_sizes[{}]={}'.format(i//n_horiz,v_sizes[i//n_horiz]))
        img_grid.paste(img, (h_sizes[i % n_horiz], v_sizes[i // n_horiz]))
    return img_grid

def pil_grid_GPS(images, FD_W,FD_H, Yaw_rad, UTM, FD_UTM, max_horiz=np.iinfo(int).max):
    img_grid = Image.new('RGB', (FD_W, FD_H), color='white')
    for i, img in enumerate(images):
        print('FD=%dx%d'%(FD_W, FD_H),'int((UTM[i][0]-FD_UTM[0])*ppm)=',int((UTM[i][0]-FD_UTM[0])*ppm),'int((UTM[i][1]-FD_UTM[2])*ppm)=',int((UTM[i][1]-FD_UTM[2])*ppm))
        img_grid.paste(img, (int((UTM[i][0]-FD_UTM[0])*ppm), int((UTM[i][1]-FD_UTM[2])*ppm)))
    return img_grid

# calculate the field size in Easting\Northing
def CalculateFDsize(src_dir):
    # parse the filenames for Lat,Lon
    LLs = ParseFile.ExtractLL(src_dir)
    print('len(LLs)=',len(LLs))
    
    UTM = []
    E_min = N_min = 10000000000
    E_max = N_max = 0
    Yaw_rad = []
    print('Calculating canvas size...')
    for i,LL in enumerate(LLs): 
        # convert LL->UTM
        Lon_deg,Lat_deg,E,N = LL2UTM.Convert_LL2UTM(LL)
        E_max = max(E_max,E)
        E_min = min(E_min,E)
        N_max = max(N_max,N)
        N_min = min(N_min,N)
        UTM.append([E,N])

        # calculate the heading angle
        LL2 = (Lat_deg, Lon_deg)
        if i==0: LL1 = LL2
        Yaw_rad.append(Heading.Calculate_Heading(LL1,LL2))
        print('%d: %s, %s'%(i+1,LL1,LL2),'yaw=%.3f'%degrees(Yaw_rad[i]))
        LL1 = LL2 
    return E_min,E_max,N_min,N_max,Yaw_rad,UTM

# check the images if tagged '-L' & '-R' for column image stitching    
def Check4WheatImg(src_dir):
    flag_shifting = False
    arr_filenames = np.empty((6, 8), dtype=object)
    for i,filename in enumerate(os.listdir(src_dir)):
        if filename.find('_L') or filename.find('_R'): 
            #print(i//8,i%8,filename)
            arr_filenames[i//8][i%8] = filename
        if filename.find('-L') or filename.find('-R'):
            flag_shifting = True
            #break    
    return flag_shifting,arr_filenames

def Stitch(Im,src_dir,overlap,skew,Heading,scale,Trim,flag_shifting=None):
    
    #Im = load_images(src_dir)

    # deskew the images
    if skew != 0: Im = deskew_images(Im,skew)

    # convert image cv2->PIL 
    #img = Image.fromarray(cv2.cvtColor(Im[0], cv2.COLOR_BGR2RGB))
    #img.show()
    
    ComboImg = []
    (ImH,ImW),im_cnt = Im[0].shape[:2],len(Im)
    print ('ImH x ImW =',ImH,ImW,'im_cnt=',im_cnt)
    print('Cropping the image...') # by removing the overlapped portion
    
    # reorder the column images if tagged '-L' & '-R' for Heading='W'    
    if flag_shifting: 
        Im = [Im[i] for i in [4, 5, 2, 3, 0, 1]]
        dst_dir = src_dir[:src_dir.find(src_dir.split('/')[-2])]+'Stitched/'    #Find parent's directroy in 1 step
    else:
        dir_str = src_dir.split('/')
        for i,st in enumerate(dir_str):
            if dir_str[i][:6].isnumeric():
                date = dir_str[i][:6]
        date_meta = date+'_%ddeg%ssize_%d%%%s%d'%(skew,scale,overlap*100,Heading[0],Heading[1])
        dst_dir = src_dir[:src_dir.find(str(date))]+'Stitched/%s/'%date_meta    #Find parent's directroy in 1 step
    #dst_dir = path+'Data/Stitch/'
    print('src_dir=',src_dir,'dst_dir=',dst_dir)
            
    print('len(Im)=',len(Im))    
    if Heading[0]=='N' or Heading[0]=='W': Im = reversed(Im)
    for i,im in enumerate (Im):
        im_crop = im[:ImH,:ImW]
        if Heading[0]=='N': # reverse order
            if Heading[1]==1: # keeping 1st image
                if i<(im_cnt-1): im_crop = im[:int(ImH*(1-overlap)),:ImW]
            elif Heading[1]==2: # keeping 2nd image
                if i!=0:    im_crop = im[int(ImH*overlap):,:ImW]        
        elif Heading[0]=='E': 
            if Heading[1]==1: # keeping 1st image
                if i!=0:    im_crop = im[:ImH,int(ImW*overlap):]
            elif Heading[1]==2: # keeping 2nd image
                if i<(im_cnt-1): im_crop = im[:ImH,:int(ImW*(1-overlap))]
        elif Heading[0]=='S':
            if Heading[1]==1: # keeping 1st image
                if i!=0:    im_crop = im[int(ImH*overlap):,:ImW]
            elif Heading[1]==2: # keeping 2nd image
                if i<(im_cnt-1): im_crop = im[:int(ImH*(1-overlap)),:ImW]
        elif Heading[0]=='W':   # reverse order 
            if Heading[1]==1: # keeping 1st image
                if i<(im_cnt-1): im_crop = im[:ImH,:int(ImW*(1-overlap))]
            elif Heading[1]==2: # keeping 2nd image
                if i!=0:    im_crop = im[:ImH,int(ImW*overlap):]
        else: messagebox.showerror(title='Error',message='Heading must be N, E, S, or W only!')
        
        # trim left and right margin
        if flag_shifting and (Trim[0] != 0 or Trim[1] != 0): 
            im_crop = im_crop[:ImH,int(ImW*Trim[0]/100):int(ImW*(100-Trim[1])/100)]

        # resize the cropped image if too big
        if scale!=1: im_crop = cv2.resize(im_crop, (int(im_crop.shape[1]*scale), int(im_crop.shape[0]*scale))) 
        print('resized WxH=',im_crop.shape)

        #plt.subplot(121),plt.imshow(im),plt.title('Input')
        #plt.subplot(122),plt.imshow(im_crop),plt.title('Output')
        #plt.show()
        
        # add a border to the image
        im_crop= cv2.copyMakeBorder(im_crop,5,5,5,5,cv2.BORDER_CONSTANT,value=(255,255,255))
        #im_crop= cv2.copyMakeBorder(im_crop[10:-10,10:-10],10,10,10,10,cv2.BORDER_CONSTANT,value=(255,255,255))
        
        img = Image.fromarray(cv2.cvtColor(im_crop, cv2.COLOR_BGR2RGB))
        ComboImg.append(img)

    #ComboImg[0].show()
    
    ## Calculate the field size
    #E_min,E_max,N_min,N_max,Yaw_rad,UTM = CalculateFDsize(src_dir)
    #FD_UTM = (E_min,E_max,N_min,N_max)
    ## convert the field size meter -> pixel
    #FD_W = int((E_max - E_min) * ppm) + imW*2
    #FD_H = int((N_max - N_min) * ppm) + imH*2
    #print('E=%.1f~%.1f, N=%.1f~%.1f'%(E_min,E_max,N_min,N_max),'W x H = %d x %d'%(FD_W,FD_H))

    # calculate the field size in pixel
    (imW,imH) = ComboImg[0].size
    FD_H, FD_W = 0, 0
    print('len(ComboImg)=',len(ComboImg))
    if Heading[0] == 'N' or Heading[0] == 'S':
        max_horiz, FD_W = 1, imW
        for i in range(im_cnt): FD_H = FD_H + ComboImg[i].size[1]
    elif Heading[0] == 'E' or Heading[0] == 'W':        
        max_horiz, FD_H = im_cnt, imH
        for i in range(im_cnt): FD_W = FD_W + ComboImg[i].size[0]
    print('W x H x cnt= %d x %d x %d'%(imW,imH,im_cnt),'=> W x H = %d x %d'%(FD_W,FD_H))
        
    # Combine images vertically
    pil_ComboImg_combo = pil_grid(ComboImg, FD_W,FD_H, max_horiz=max_horiz)
    # Combine images on GPS loc
    #pil_ComboImg_combo = pil_grid_GPS(ComboImg, FD_W,FD_H, Yaw_rad, UTM, FD_UTM,max_horiz=2)

    # save the stitched image    
    if not os.path.exists(dst_dir): os.makedirs(dst_dir)
    fdir = src_dir[src_dir.find(src_dir.split('/')[-1]):]    #Find file directroy            
    print('src_dir.split('')[-1]=',src_dir.split('/')[-1])
    print('src_dir=',src_dir,'dst_dir=',dst_dir,'fdir=',fdir)
    if flag_shifting: 
        pil_ComboImg_combo.save(dst_dir+'%s_%d&%d%%%s%d.tif'%(fdir,Trim[0],Trim[1],Heading[0],Heading[1]),'TIFF') 
    else:    
        if fdir.find(date)!=-1: fdir = fdir[fdir.find(date)+len(date)+1:]
        print('date=',date,'fdir=',fdir)
        pil_ComboImg_combo.save(dst_dir+'%s_%s.tif'%(date,fdir),'TIFF') 
    return dst_dir

if __name__ == '__main__':
    root = tk.Tk()
    root.title("iStitch") # change the title of dialog widget        
    root.lift() # bring a dialog in front
        
    # load images from a folder
    #src_dir = path+'DAQ2/Nadir/021921/'
    #src_dir = tk.filedialog.askdirectory() 
    #if len(src_dir)==0: exit()
    
    # define default inputs 
    stage = 'Early' # default growth stage to row detection algorithm
    skew_GC = 30       # default 30 deg skew angle in Geo-Calibration
    skew = 0       # default 0 deg skew angle
    Heading = ['N',1]   # default heading direction to stitching, image keeping on the 1st image
    Shifting = ['E',1]   # default Path Shifting direction to stitching, image keeping on the 1st image
    overlap = 22    # default 22% overlap between images
    Trim = [30,30]    # default trim 30% in left and right
    scale = .5       # default scale factor 0.5
    defaultInputs = [stage,skew_GC,skew,Heading,Shifting,overlap,Trim,scale]

    # update variable inputs
    d = Dialog_Input4Stitch(root,defaultInputs)
    if d.skew == None: exit()
    GeoCal_var = d.GeoCal_var
    stage = d.stage
    skew_GC = d.skew_GC
    skew = d.skew
    Heading = [d.heading,d.headingKeep]    
    Shifting_var = d.Shift_var
    Shifting = [d.shifting,d.shiftingKeep]    
    overlap = d.overlap/100.0   # % -> decimal
    Trim = d.trim   # left & right
    scale = d.scale
    print('GeoCal_var=',GeoCal_var,'stage=',stage,'skew_GC=',skew_GC,'skew=',skew,'Heading=',Heading,'Shifting_var=',Shifting_var,'Shifting=',Shifting,'overlap=',overlap,'Trim=',Trim,'scale=',scale)
    start_time = time.time()
    if len(src_dir)==0: 
        print('src_dir not selected!')
        exit()    

    # Geometric calibration for row alignment using Row Detection    
    if GeoCal_var: 
        print('skew_GC=',skew_GC,'stage=',stage)
        dst_dir = RD.main(src_dir,skew=skew_GC,stage=stage)  # growth stage [Early,Mid,Late] for row detection
        print('dst_dir',dst_dir)
        src_dir = dst_dir
        
    # list subfolders
    subfolders = [ f.path for f in os.scandir(src_dir) if f.is_dir() ]
    #print('src_dir=',src_dir,'subfolders=',subfolders)
    for i in range(len(subfolders)): subfolders[i] = subfolders[i].replace('\\','/')
    #print('src_dir=',src_dir,'subfolders=',subfolders)

    if len(subfolders)>0:
        for i,im_dir in enumerate(subfolders):
            print(i,im_dir)
            Im = load_images(im_dir)
            dst_dir= Stitch(Im,im_dir,overlap,skew,Heading,scale,Trim=[0,0])
    else: 
        flag_shifting,arr_filenames = Check4WheatImg(src_dir) # check if images tagged '-L' & '-R' for wheat 
        #print('arr_filenames=',arr_filenames)
        for filenames in arr_filenames:
            Im = []
            for filename in filenames:
                #print('filename=',filename)
                im = cv2.imread(os.path.join(src_dir,filename))
                Im.append(im)
            im_dir = src_dir+filename[6:11]
            #print('len(Im)=',len(Im),'im_dir=',im_dir)
            dst_dir = Stitch(Im,im_dir,overlap,skew,Heading,scale,Trim=[0,0])
    
    src_dir = dst_dir[:-1]
    print('src_dir=',src_dir)
  
    # define default inputs for stitching shift
    if Shifting_var == 1:
        overlap = skew = 0  # default 0% overlap, 0 deg skew angle
        scale = 1       # default scale factor
        #Trim = [0,0]    # default trim % in left and right
        print('overlap=',overlap,'skew=',skew,'Shifting=',Shifting,'scale=',scale,'Trim=',Trim)
    
        flag_shifting,arr_filenames = Check4WheatImg(src_dir) # check if images tagged '-L' & '-R' for wheat 
        Im = load_images(src_dir)
        dst_dir = Stitch(Im,src_dir,overlap,skew,Shifting,scale,Trim,flag_shifting)

    print("Total program run time = %.3f seconds." % (time.time() - start_time))
